package pl.net.linux.krzak.kchat.util;
/*****************************/
/* kChat v2.0pre4	     */
/* krzak@linux.net.pl        */
/* Marcin Krzyzanowski       */
/* http://krzak.linux.net.pl */
/* License: GPL              */
/*****************************/
/* Copyright (C) 2001  Marcin Krzyzanowski <krzak@linux.net.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class kScrollbar extends Component implements Adjustable {
    AdjustmentListener adjustmentListener;
    int orientation = java.awt.Scrollbar.VERTICAL;
    int value, minimum, maximum, visible, block, increment;
    private boolean pressed = false;
    private int plusminus = 1;
    private boolean SELECTED_UP = false;
    private boolean SELECTED_DOWN = false;
    Image offscreen;
    
    
    public kScrollbar(int _orientation, int _value, int _visible, int _minimum, int _maximum) {
    this.orientation = _orientation;
    this.value = _value;
    this.minimum = _minimum;
    this.maximum = _maximum;
    this.visible = _visible; // Just for compatibility
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    }
    
    public kScrollbar(int _orientation) { 
    this(_orientation,0,1,0,100); 
    }
    
    public kScrollbar() {
    this(java.awt.Scrollbar.VERTICAL);
    }

    public int getOrientation() {
    return(this.orientation);
    }
    
    public int getValue() {
    return(this.value);
    }

    public void setValue(int newValue) {
    if ((newValue>=minimum) && (newValue<=maximum))
    this.value = newValue;
    invalidate();
    this.paint(this.getGraphics());
    }
    
    public void addAdjustmentListener(AdjustmentListener listener) {
    adjustmentListener = AWTEventMulticaster.add(adjustmentListener, listener);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    }

    public void removeAdjustmentListener(AdjustmentListener listener) {
    adjustmentListener = AWTEventMulticaster.remove(adjustmentListener, listener);
    }
    
    public int getBlockIncrement() {
    return(this.block);
    }

    public void setBlockIncrement(int _block) {
    this.block = _block;
    }
    
    public int getMaximum() {
    return(maximum);
    }
    
    public void setMaximum(int _max) {
    this.maximum = _max;
    }

    public int getMinimum() {
    return(minimum);
    }
    
    public void setMinimum(int _min) {
    this.minimum = _min;
    }

    public void setUnitIncrement(int u) {
    this.increment = u;
    }
    
    public int getUnitIncrement() {
    return(increment);
    }
    
    public void setVisibleAmount(int v) {
    this.visible = v;
    }
    
    public int getVisibleAmount() {
    return(visible);
    }
    
    public Dimension getMinimumSize() {
    return new Dimension(10, 30);
    }

    public Dimension getPreferredSize() {
    return new Dimension(10, (maximum)+20); // for VERTICAL ONLY
    }


   public void processMouseEvent(MouseEvent e) {
       switch(e.getID()) {
          case MouseEvent.MOUSE_PRESSED:
	    pressed = true;
	    if ((e.getY()>0)&&(e.getY()<15)) { plusminus = 1; SELECTED_UP = true; }
	    if ((e.getY()>(getSize().height-15))&&(e.getY()<(getSize().height))) { plusminus = -1; SELECTED_DOWN = true; }
	    this.paint(this.getGraphics()); 
	    break;
          case MouseEvent.MOUSE_RELEASED:
	    if(adjustmentListener != null) {
		int newValue = value + (1*plusminus);
		if ((newValue>=minimum) && (newValue<=maximum)) value = newValue;
	       adjustmentListener.adjustmentValueChanged(new AdjustmentEvent(
		   this, AdjustmentEvent.ADJUSTMENT_FIRST, 
		   AdjustmentEvent.TRACK,(this.value)));
	    }
	    if(pressed == true) {
		pressed = false;
		SELECTED_UP=false;
		SELECTED_DOWN=false;
		this.paint(this.getGraphics());
	    }
	    break;
          case MouseEvent.MOUSE_ENTERED:
	    break;
          case MouseEvent.MOUSE_EXITED:
	    if(pressed == true) {
		pressed = false;
		SELECTED_UP=false;
		SELECTED_DOWN=false;
		this.paint(this.getGraphics());
	    }
	    break;
       }
       super.processMouseEvent(e);
   }

    public void invalidate() {
	super.invalidate();
	offscreen = null;
    }
    
    public void update (Graphics g) { paint(g); }

    public void paint(Graphics g) {
    if (offscreen == null) { offscreen = createImage(getSize().width, getSize().height); }
    Graphics og = offscreen.getGraphics();
    switch (orientation) {
	case Scrollbar.VERTICAL:
	    og.setColor(getBackground());
	    og.fillRect(0,0,getSize().width,getSize().height);
	    
	    og.setColor(getForeground().darker().darker());
	    og.fill3DRect(0,0,getSize().width-1,getSize().height-1,false);
	    
	    // ARROWS
	    if (SELECTED_UP) og.setColor(getBackground().darker());	
		else og.setColor(getBackground());	    
	    og.fill3DRect(0,0,getSize().width-1,15,true); // UP
	    if (SELECTED_DOWN) og.setColor(getBackground().darker());
		else og.setColor(getBackground());	    
	    og.fill3DRect(0,getSize().height-15,getSize().width-1,15,true); // DOWN
	    
	    int s = ((getSize().width-1)/2);
	    
	    og.setColor(getForeground()); 
	    // UPPER ARROW
	    og.drawLine(s,2 ,s,12);
	    og.drawLine(s-2,6 ,s,2);      og.drawLine(s+2,6 ,s,2);
	    
	    // LOWER ARROW
	    int h = getSize().height-15;
	    og.drawLine(s,2+h ,s,12+h);
	    og.drawLine(s-2,8+h ,s,12+h); og.drawLine(s+2,8+h ,s,12+h);
	break;
	}
    super.paint(og);
    g.drawImage(offscreen,0,0,null);
    og.dispose();
    }

   public boolean contains(int x, int y) {
    int xm = getSize().width;
    int ym = getSize().height;
    return((x>0)&&(x<xm)&&(y>0)&&(y<ym));
   }


}
